<?php

namespace Drupal\change_labels\EventSubscriber;

use Drupal\Core\Render\Element;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\field_event_dispatcher\Event\Field\FieldWidgetThirdPartySettingsFormEvent;
use Drupal\field_event_dispatcher\Event\Field\WidgetCompleteFormAlterEvent;
use Drupal\field_event_dispatcher\FieldHookEvents;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Change labels event subscriber.
 */
class ChangeAddAnotherLabelSubscriber implements EventSubscriberInterface {

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      FieldHookEvents::FIELD_WIDGET_THIRD_PARTY_SETTINGS_FORM => ['addSettings'],
      FieldHookEvents::WIDGET_COMPLETE_FORM_ALTER => ['alterForm'],
    ];
  }

  /**
   * Event handler for the third party settings of a widget.
   *
   * @param \Drupal\field_event_dispatcher\Event\Field\FieldWidgetThirdPartySettingsFormEvent $event
   *   Triggering event.
   */
  public function addSettings(FieldWidgetThirdPartySettingsFormEvent $event) {
    if ($event->getFieldDefinition()->getFieldStorageDefinition()->getCardinality() === 1) {
      return;
    }

    $element = [];
    $element['add_another'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Button label for "Add another"'),
      '#default_value' => $event->getPlugin()->getThirdPartySetting('change_labels', 'add_another'),
    ];
    $element['hide_add_another'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Hide "Add another"'),
      '#default_value' => $event->getPlugin()->getThirdPartySetting('change_labels', 'hide_add_another'),
    ];
    $element['force_single_cardinality'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Force the widget to allow only one item'),
      '#default_value' => $event->getPlugin()->getThirdPartySetting('change_labels', 'force_single_cardinality'),
    ];
    $event->addElements('change_labels', $element);
  }

  /**
   * Event handler for altering a widget.
   *
   * @param \Drupal\field_event_dispatcher\Event\Field\WidgetCompleteFormAlterEvent $event
   *   Triggering event.
   */
  public function alterForm(WidgetCompleteFormAlterEvent $event) {
    $addAnotherLabel = $event->getContext()['widget']->getThirdPartySetting('change_labels', 'add_another');
    $elements = &$event->getElements();
    if ($addAnotherLabel && isset($elements['add_more'])) {
      $elements['add_more']['#value'] = $addAnotherLabel;
    }
    $hideAddAnother = $event->getContext()['widget']->getThirdPartySetting('change_labels', 'hide_add_another');
    if ($hideAddAnother && isset($elements['add_more'])) {
      $elements['add_more']['#access'] = FALSE;
    }
    $singleCardinality = $event->getContext()['widget']->getThirdPartySetting('change_labels', 'force_single_cardinality');
    if ($singleCardinality && isset($elements['#theme']) && $elements['#theme'] === 'field_multiple_value_form') {
      $elements['#cardinality'] = 1;
      $elements['#cardinality_multiple'] = FALSE;
      $children = Element::children($elements);
      foreach ($children as $key) {
        $elements[$key]['_weight']['#access'] = FALSE;
      }
    }

  }

}
