<?php

namespace Drupal\change_labels\EventSubscriber;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\field_event_dispatcher\Event\Field\FieldWidgetThirdPartySettingsFormEvent;
use Drupal\field_event_dispatcher\Event\Field\WidgetCompleteFormAlterEvent;
use Drupal\field_event_dispatcher\FieldHookEvents;
use Drupal\file\Plugin\Field\FieldWidget\FileWidget;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Change labels event subscriber.
 */
class ChangeRemoveLabelSubscriber implements EventSubscriberInterface {

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      FieldHookEvents::FIELD_WIDGET_THIRD_PARTY_SETTINGS_FORM => ['addSettings'],
      FieldHookEvents::WIDGET_COMPLETE_FORM_ALTER => ['alterForm'],
    ];
  }

  /**
   * Event handler for the third party settings of a widget.
   *
   * @param \Drupal\field_event_dispatcher\Event\Field\FieldWidgetThirdPartySettingsFormEvent $event
   *   Triggering event.
   */
  public function addSettings(FieldWidgetThirdPartySettingsFormEvent $event) {
    if (!($event->getPlugin() instanceof FileWidget)) {
      return;
    }

    $element = [];
    $element['remove_label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Button label for "Remove"'),
      '#default_value' => $event->getPlugin()->getThirdPartySetting('change_labels', 'remove_label'),
    ];
    $event->addElements('change_labels', $element);
  }

  /**
   * Event handler for altering a widget.
   *
   * @param \Drupal\field_event_dispatcher\Event\Field\WidgetCompleteFormAlterEvent $event
   *   Triggering event.
   */
  public function alterForm(WidgetCompleteFormAlterEvent $event) {
    $settings = $event->getContext()['widget']->getThirdPartySetting('change_labels', 'remove_label');
    if ($settings) {
      $elements = &$event->getElements();
      foreach (Element::children($elements) as $child) {
        $elements[$child]['#change_label_remove'] = $settings;
        $elements[$child]['#process'][] = [
          static::class,
          'processChangeRemoveLabel',
        ];
      }
    }
  }

  /**
   * Process callback to swap default label for the configured one.
   */
  public static function processChangeRemoveLabel($element, FormStateInterface $form_state, $form) {
    if (isset($element['remove_button'])) {
      $element['remove_button']['#value'] = $element['#change_label_remove'];
    }
    return $element;
  }

}
